/*
 * Copyright (c) 2001-2010, Purdue University
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  * Redistributions of source code must retain the above copyright
 *  *    notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *  * Neither the name of the Purdue University nor the
 *    names of its contributors may be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

package immortal.persistentScope.transientScope;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

/**
 * Represents a definite collision that has occured.
 * @author Filip Pizlo
 */
class Collision {
	
	private Aircraft _one, _two;

	/** The location where the collision happened. */
	private Vector3d location;



	/** Construct a Coollision with two aircraft an a location. */
	public Collision(Aircraft one, Aircraft two, Vector3d location) {
	
		this.location = location;
		_one=one;
		_two=two;
	}

	public boolean hasAircraft(Aircraft a) {
		if (_one.equals(a)) return true;
		if (_two.equals(a)) return true;
		return false;
	}
	public Aircraft first() { return _one; }
	public Aircraft second() { return _two; }

	
	
	/** Returns the location of the collision. You are not to modify this location. */
	public Vector3d getLocation() { return location; }

	/** Returns a hash code for this object. It is based on the hash codes of the aircraft. */

	public int hashCode() {
		int ret = 0;
		
		ret+=_one.hashCode();
		ret+=_two.hashCode();
		return ret;
	}

	/** Determines collision equality. Two collisions are equal if they have the same aircraft.*/

	public boolean equals(Object _other) {
		if (_other == this)  return true;
		if (!(_other instanceof Collision)) return false;
		Collision other = (Collision) _other;
	
		if (_one!=other._one) return false;
		if (_two!=other._two) return false;
		return true;
	}

	/** Returns a helpful description of this object. */

	public String toString() {
		StringBuffer buf = new StringBuffer("Collision between ");
		boolean first = true;
		
		buf.append(_one.toString()+", "+_two.toString());
		buf.append(" at "); buf.append(location.toString());
		return buf.toString();
	}
}
